package com.bernhardriedl.projects.henonattractor.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.UIManager;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeListener;

import com.bernhardriedl.projects.henonattractor.HenonAttractor;
import com.bernhardriedl.projects.henonattractor.images.ImagesContainer;

/**
 * Tools Library 2.10
 * https://www.bernhard-riedl.com/projects/
 */

import com.bernhardriedl.projects.tools.gui.AboutDialog;
import com.bernhardriedl.projects.tools.gui.EnhancedButton;

public class MainMenu
	extends JFrame
	implements ActionListener, CaretListener, ChangeListener {
	
	private static final long serialVersionUID = 4979485336450167818L;
	
	/**
	 * GUI components
	 */
	
	private JPanel content = new JPanel();
	
	private JLabel labelA = new JLabel();
	private JLabel labelB = new JLabel();
	
	private JTextField textfieldA = new JTextField();
	private JTextField textfieldB = new JTextField();
	
	private JSlider sliderA = new JSlider();
	private JSlider sliderB = new JSlider();
	
	private JPanel innerContent = new JPanel();
	
	private EnhancedButton display = new EnhancedButton();
	private EnhancedButton animate=new EnhancedButton();

	private JMenuBar menubar = new JMenuBar();
	private JMenu menu_file = new JMenu();
	private JMenu menu_help = new JMenu();

	private JMenuItem menuitem_exit = new JMenuItem();

	private JMenuItem menuitem_about = new JMenuItem();
	
	private JToolBar toolbar=new JToolBar(HenonAttractor.NICENAME);
	
	private boolean updateTextA=false;
	private boolean updateTextB=false;
	
	private boolean updateSliderA=false;
	private boolean updateSliderB=false;
	
	/**
	 * Constructor
	 */
	
	public MainMenu() {
		
		try {
			ImagesContainer.loadPics();
			
			this.setTitle(HenonAttractor.NICENAME);
			
			this.setIconImage(ImagesContainer.icon.getImage());
			
			this.makeComponents();
			this.setContentPane(content);
			
			this.makeMenu();
			this.setJMenuBar(menubar);
			
			this.getRootPane().setDefaultButton(display);
			
			this.setSize(500, 160);
			com.bernhardriedl.projects.tools.gui.Utils.centerWindow(this);
			
			this.setVisible(true);
		}
		
		catch (Exception e) {
			JOptionPane.showMessageDialog(this, e.getMessage());
			e.printStackTrace();
		}
	}
	
	/**
	 * Initialize GUI components
	 * @throws Exception
	 */
	
	private void makeComponents() throws Exception {
		
		this.content.setLayout(new BorderLayout());
		
		this.labelA.setText("1.00 <= A <= 1.42");
		this.labelB.setText("0.01 <= B <= 0.50");
		
		this.textfieldA.setText("1.4");
		this.textfieldB.setText("0.3");
		
		this.textfieldA.setForeground(Color.white);
		this.textfieldB.setForeground(Color.white);
		
		this.textfieldA.setBackground(Color.black);
		this.textfieldB.setBackground(Color.black);
		
		this.textfieldA.addCaretListener(this);
		this.textfieldB.addCaretListener(this);
		
		this.sliderA.setPaintTicks(true);
		this.sliderA.setMajorTickSpacing(5);
		this.sliderA.setMinorTickSpacing(1);
		this.sliderA.setSnapToTicks(true);
		
	    this.sliderB.setPaintTicks(true);
	    this.sliderB.setMajorTickSpacing(5);
	    this.sliderB.setMinorTickSpacing(1);
	    this.sliderB.setSnapToTicks(true);
	    
	    this.sliderA.setMinimum(100);
		this.sliderA.setMaximum(142);
		
		this.sliderB.setMinimum(1);
		this.sliderB.setMaximum(50);
		
		this.sliderA.setValue(140);
		this.sliderB.setValue(30);
		
		this.sliderA.addChangeListener(this);
		this.sliderB.addChangeListener(this);
		
		this.display.setText("display");
		this.display.addActionListener(this);
		
		this.animate.setText("animate");
		this.animate.addActionListener(this);
		
		this.toolbar.add(display);
		this.toolbar.addSeparator();
		this.toolbar.add(animate);
		
		this.innerContent.setLayout(new GridLayout(2,0));
		this.innerContent.setBorder(BorderFactory.createEmptyBorder(7, 7, 7, 7));
		
		/**
		 * first line
		 */
		
		this.innerContent.add(labelA);
		this.innerContent.add(sliderA);
		this.innerContent.add(textfieldA);
		
		/**
		 * second line
		 */
		
		this.innerContent.add(labelB);
		this.innerContent.add(sliderB);
		this.innerContent.add(textfieldB);
		
		this.content.add(this.innerContent,  BorderLayout.CENTER);
		this.content.add(this.toolbar,  BorderLayout.SOUTH);
		
		this.updateTextA=true;
		this.updateTextB=true;
		
		this.updateSliderA=true;
		this.updateSliderB=true;
	}
	
	/**
	 * Initialize menu
	 * @throws Exception
	 */
	
	private void makeMenu() throws Exception {
		menuitem_exit.setIcon(ImagesContainer.iconStop16);
		menuitem_exit.setText("Exit");
		menuitem_exit.addActionListener(this);
		menuitem_exit.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F4, java.awt.event.KeyEvent.ALT_MASK, false));
		
		menuitem_about.setIcon(ImagesContainer.iconAbout16);
		menuitem_about.setText("About");
		menuitem_about.addActionListener(this);
		menuitem_about.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F1, 0, false));
		
		menu_file.setText("File");
		menu_file.setMnemonic('F');
		menu_file.add(menuitem_exit);
		
		menu_help.setText("Help");
		menu_help.add(menuitem_about);
		menu_help.setMnemonic('H');
		
		menubar.add(menu_file);
		menubar.add(Box.createHorizontalGlue());
		menubar.add(menu_help);
	}
	
	/**
	 * actionPerformed
	 * @param e ActionEvent
	 */
	
	public void actionPerformed (ActionEvent e) {
		try {
			if (e.getSource().equals(this.display)) {
				
				double a=Double.parseDouble(textfieldA.getText());
				double b=Double.parseDouble(textfieldB.getText());

				/**
				 * check limits
				 */
				
				if (a<1.0 | a>1.42)
					JOptionPane.showMessageDialog(this, "The value for a is out of range", "Value out of range!", JOptionPane.ERROR_MESSAGE);
				else if (b<0.01 | b>0.5)
					JOptionPane.showMessageDialog(this, "The value for b is out of range.", "Value out of range!", JOptionPane.ERROR_MESSAGE);
				
				/**
				 * if the limits are ok,
				 * we create a new
				 * non-animation IteratorContainer
				 */
				else
					new IteratorContainer(a, b, false, this);
			}
			
			else if (e.getSource().equals(this.menuitem_about))
				this.showabout();
			
			else if (e.getSource().equals(this.animate))
				this.startAnimation();
			
			else if (e.getSource().equals(this.menuitem_exit))
				com.bernhardriedl.projects.tools.gui.Utils.exitProgramm(this);
		}
		
    /**
     * input error
     */
		
		catch (Exception ex) {
			JOptionPane.showMessageDialog(this, "Input-Error", "Error!", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	/**
	 * caretUpdate -
	 * update slider if content of textfield has been updated
	 * @param e CaretEvent  
	 */
	
	public void caretUpdate(CaretEvent e) {
		
		/**
		 * handle textfieldA
		 */
		
		if (e.getSource().equals(this.textfieldA) && this.textfieldA.getText().length()>0 && this.updateTextA==true) {
			try {
				int value=(int) (Double.parseDouble(this.textfieldA.getText())*100);
				
				/**
				 * check for limits
				 * and update slider
				 */
				
				if (value>99 && value<143) {
					this.updateSliderA=false;
					this.sliderA.setValue((int) value);
					this.updateSliderA=true;
				}
			}
			
			catch(Exception ex) {
				
				/**
				 * do nothing
				 */
				
			}
		}
		
		/**
		 * handle textfieldB
		 */
		
		else if (e.getSource().equals(this.textfieldB) && this.textfieldB.getText().length()>0 && this.updateTextB==true) {
			try {
				int value=(int) (Double.parseDouble(this.textfieldB.getText())*100);
				
				/**
				 * check for limits
				 * and update slider
				 */
				
				if (value>0 && value<51) {
					this.updateSliderB=false;
					this.sliderB.setValue((int) value);
					this.updateSliderB=true;
				}
			}
			
			catch(Exception ex) {
				
				/**
				 * do nothing
				 */
				
			}
		}
	}
	
	/**
	 * stateChanged -
	 * update textfield if position of slider has been changed
	 * @param e ChangeEvent  
	 */
	
	public void stateChanged(javax.swing.event.ChangeEvent e) {
		
		/**
		 * handle sliderA
		 */
		
		if (e.getSource().equals(this.sliderA) && this.updateSliderA==true) {
			double value=(double) this.sliderA.getValue()/100;
			
			this.updateTextA=false;
			this.textfieldA.setText(String.valueOf(value));
			this.updateTextA=true;
		}
		
		/**
		 * handle sliderB
		 */
		
		if (e.getSource().equals(this.sliderB) && this.updateSliderB==true) {
			double value=(double) this.sliderB.getValue()/100;
			
			this.updateTextB=false;
			this.textfieldB.setText(String.valueOf(value));
			this.updateTextB=true;
		}
	}
	
	/**
	 * displays the about-window
	 */
	
	private void showabout() {
		Vector <String> text=new Vector <String> ();
		
		text.addElement("The Hnon map is a discrete-time dynamical system.");
		text.addElement("It is one of the most studied examples of");
		text.addElement("dynamical systems that exhibit chaotic behavior.");

		Vector <String> urls=new Vector <String> ();
		urls.addElement("https://en.wikipedia.org/wiki/Henon_attractor");
		
	    new AboutDialog(
	    	this,
	    	HenonAttractor.NICENAME,
	    	HenonAttractor.VERSION,
	    	"2002-2012",
	    	"Dr. Bernhard Riedl",
	    	text,
	    	urls,
	    	385,
	    	180
	    );
	}
	
	/**
	 * start the animation
	 */
	
	private void startAnimation() {
		new IteratorContainer(1.00, 0.3, true, this);
	}
	
	/**
	 * setLookAndFeel
	 */

	static {

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}

		catch(Exception ex) {

			/**
			 * do nothing
			 */

		}
	}

	/**
	 * override processWindowEvent
	 * to allow for a 'Are you sure'-prompt
	 * @param e WindowEvent
	 */

	protected void processWindowEvent(WindowEvent e) {
		if (e.getID() == WindowEvent.WINDOW_CLOSING)
			com.bernhardriedl.projects.tools.gui.Utils.exitProgramm(this);
	}
}